@@ -5,6 +5,7 @@
 
 #include "identity.h"
 #include "post.h"
+#include "preferences.h"
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QObject>
@@ -14,6 +15,7 @@ class Notification;
 class QNetworkReply;
 class QHttpMultiPart;
 class QFile;
+class Preferences;
 
 class AbstractAccount : public QObject
 {
@@ -25,10 +27,11 @@ class AbstractAccount : public QObject
     Q_PROPERTY(int maxPostLength READ maxPostLength NOTIFY fetchedInstanceMetadata)
     Q_PROPERTY(QString instanceName READ instanceName NOTIFY fetchedInstanceMetadata)
     Q_PROPERTY(QUrl authorizeUrl READ getAuthorizeUrl NOTIFY registered)
-    Q_PROPERTY(Identity *identity READ identityObj CONSTANT)
+    Q_PROPERTY(Identity *identity READ identity NOTIFY identityChanged)
+    Q_PROPERTY(Preferences *preferences READ preferences CONSTANT)
 
 public:
-    AbstractAccount(QObject *parent, const QString &name, const QString &instanceUri);
+    AbstractAccount(QObject *parent, const QString &instanceUri);
     AbstractAccount(QObject *parent);
 
     /// Register the application to the mastodon server
@@ -42,10 +45,11 @@ public:
     Q_INVOKABLE QUrl getAuthorizeUrl() const;
     QUrl getTokenUrl() const;
     Q_INVOKABLE void setToken(const QString &authcode);
-    Q_INVOKABLE Post *newPost();
     bool haveToken() const;
     virtual void validateToken() = 0;
 
+    Preferences *preferences() const;
+
     // name
     QString username() const;
     void setUsername(const QString &name);
@@ -57,24 +61,22 @@ public:
     size_t maxPostLength() const;
     QString instanceName() const;
 
-    // identity
-    const Identity &identity();
-
-    Identity *identityObj();
-    void setDirtyIdentity();
-    const std::shared_ptr<Identity> identityLookup(const QString &acct, const QJsonObject &doc);
-    bool identityCached(const QString &acct) const;
+    /// Get identity of the account
+    Identity *identity();
+    const std::shared_ptr<Identity> identityLookup(const QString &accountId, const QJsonObject &doc);
+    bool identityCached(const QString &accountId) const;
 
     // timeline
     void fetchTimeline(const QString &timelineName, const QString &from_id);
     void invalidate();
 
     // posting statuses
-    Q_INVOKABLE void postStatus(Post *p);
     void favorite(Post *p);
     void unfavorite(Post *p);
     void repeat(Post *p);
     void unrepeat(Post *p);
+    void bookmark(Post *p);
+    void unbookmark(Post *p);
 
     // uploading media
     void updateAttachment(Attachment *a);
@@ -101,14 +103,25 @@ public:
     /// \param authenticated Whether the request should be authentificated
     /// \param parent The parent object that calls get() or the callback belongs to
     /// \param callback The callback that should be executed if the request is successful
-    virtual void get(const QUrl &url, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
+    /// \param errorCallback The callback that should be executed if the request is not successful
+    virtual void get(const QUrl &url,
+                     bool authenticated,
+                     QObject *parent,
+                     std::function<void(QNetworkReply *)> callback,
+                     std::function<void(QNetworkReply *)> errorCallback = nullptr) = 0;
 
     /// Make an HTTP POST request to the mastodon server
     /// \param url The url of the request
     /// \param doc The request body as JSON
     /// \param parent The parent object that calls get() or the callback belongs to
     /// \param callback The callback that should be executed if the request is successful
-    virtual void post(const QUrl &url, const QJsonDocument &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
+    virtual void post(const QUrl &url,
+                      const QJsonDocument &doc,
+                      bool authenticated,
+                      QObject *parent,
+                      std::function<void(QNetworkReply *)> callback,
+                      std::function<void(QNetworkReply *)> errorCallback = nullptr,
+                      QHash<QByteArray, QByteArray> headers = {}) = 0;
 
     /// Make an HTTP POST request to the mastodon server
     /// \param url The url of the request
@@ -118,12 +131,13 @@ public:
     /// \param callback The callback that should be executed if the request is successful
     virtual void post(const QUrl &url, const QUrlQuery &formdata, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
 
-    virtual void post(const QUrl &url, QHttpMultiPart *message, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
+    virtual QNetworkReply *post(const QUrl &url, QHttpMultiPart *message, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
     virtual void put(const QUrl &url, const QJsonDocument &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
     virtual void patch(const QUrl &url, QHttpMultiPart *multiPart, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)>) = 0;
+    virtual void deleteResource(const QUrl &url, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback) = 0;
 
     /// Upload a file
-    virtual void upload(Post *post, QFile *file, const QString &filename) = 0;
+    virtual QNetworkReply *upload(const QUrl &filename, std::function<void(QNetworkReply *)> callback) = 0;
 
     /// Write account to settings
     virtual void writeToSettings(QSettings &settings) const = 0;
@@ -200,7 +214,7 @@ public:
 Q_SIGNALS:
     void authenticated();
     void registered();
-    void identityChanged(AbstractAccount *account);
+    void identityChanged();
     void fetchedTimeline(const QString &timelineName, QList<Post *> posts);
     void invalidated();
     void nameChanged();
@@ -220,8 +234,9 @@ protected:
     QString m_client_secret;
     size_t m_maxPostLength;
     QString m_instance_name;
-    Identity m_identity;
+    std::shared_ptr<Identity> m_identity;
     AllowedContentType m_allowedContentTypes;
+    Preferences *m_preferences = nullptr;
 
     // OAuth authorization
     QUrlQuery buildOAuthQuery() const;
