@@ -5,15 +5,19 @@
 #include "account.h"
 #include "accountmanager.h"
 #include "notificationhandler.h"
+#include "preferences.h"
 #include "tokodon_debug.h"
 #include "tokodon_http_debug.h"
+#include "utils.h"
+#include <QFileInfo>
 #include <QNetworkAccessManager>
 
-Account::Account(const QString &name, const QString &instanceUri, QNetworkAccessManager *nam, bool ignoreSslErrors, QObject *parent)
-    : AbstractAccount(parent, name, instanceUri)
+Account::Account(const QString &instanceUri, QNetworkAccessManager *nam, bool ignoreSslErrors, QObject *parent)
+    : AbstractAccount(parent, instanceUri)
     , m_ignoreSslErrors(ignoreSslErrors)
     , m_qnam(nam)
 {
+    m_preferences = new Preferences(this);
     setInstanceUri(instanceUri);
 
     auto notificationHandler = new NotificationHandler(m_qnam, this);
@@ -26,7 +30,7 @@ Account::Account(const QSettings &settings, QNetworkAccessManager *nam, QObject
     : AbstractAccount(parent)
     , m_qnam(nam)
 {
-    m_identity.reparentIdentity(this);
+    m_preferences = new Preferences(this);
     auto notificationHandler = new NotificationHandler(m_qnam, this);
     connect(this, &Account::notification, notificationHandler, [this, notificationHandler](std::shared_ptr<Notification> notification) {
         notificationHandler->handle(notification, this);
@@ -40,27 +44,40 @@ Account::~Account()
     m_identityCache.clear();
 }
 
-void Account::get(const QUrl &url, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> reply_cb)
+void Account::get(const QUrl &url,
+                  bool authenticated,
+                  QObject *parent,
+                  std::function<void(QNetworkReply *)> reply_cb,
+                  std::function<void(QNetworkReply *)> errorCallback)
 {
     QNetworkRequest request = makeRequest(url, authenticated);
     qCDebug(TOKODON_HTTP) << "GET" << url;
 
     QNetworkReply *reply = m_qnam->get(request);
     reply->setParent(parent);
-    handleReply(reply, reply_cb);
+    handleReply(reply, reply_cb, errorCallback);
 }
 
-void Account::post(const QUrl &url, const QJsonDocument &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> reply_cb)
+void Account::post(const QUrl &url,
+                   const QJsonDocument &doc,
+                   bool authenticated,
+                   QObject *parent,
+                   std::function<void(QNetworkReply *)> reply_cb,
+                   std::function<void(QNetworkReply *)> error_cb,
+                   QHash<QByteArray, QByteArray> headers)
 {
     auto post_data = doc.toJson();
 
     QNetworkRequest request = makeRequest(url, authenticated);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
+    for (const auto [headerKey, headerValue] : asKeyValueRange(headers)) {
+        request.setRawHeader(headerKey, headerValue);
+    }
     qCDebug(TOKODON_HTTP) << "POST" << url << "[" << post_data << "]";
 
     auto reply = m_qnam->post(request, post_data);
     reply->setParent(parent);
-    handleReply(reply, reply_cb);
+    handleReply(reply, reply_cb, error_cb);
 }
 
 void Account::put(const QUrl &url, const QJsonDocument &doc, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> reply_cb)
@@ -89,7 +106,7 @@ void Account::post(const QUrl &url, const QUrlQuery &formdata, bool authenticate
     handleReply(reply, reply_cb);
 }
 
-void Account::post(const QUrl &url, QHttpMultiPart *message, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> reply_cb)
+QNetworkReply *Account::post(const QUrl &url, QHttpMultiPart *message, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> reply_cb)
 {
     QNetworkRequest request = makeRequest(url, authenticated);
 
@@ -98,6 +115,7 @@ void Account::post(const QUrl &url, QHttpMultiPart *message, bool authenticated,
     QNetworkReply *reply = m_qnam->post(request, message);
     reply->setParent(parent);
     handleReply(reply, reply_cb);
+	return reply;
 }
 
 void Account::patch(const QUrl &url, QHttpMultiPart *multiPart, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback)
@@ -110,6 +128,17 @@ void Account::patch(const QUrl &url, QHttpMultiPart *multiPart, bool authenticat
     handleReply(reply, callback);
 }
 
+void Account::deleteResource(const QUrl &url, bool authenticated, QObject *parent, std::function<void(QNetworkReply *)> callback)
+{
+    QNetworkRequest request = makeRequest(url, authenticated);
+
+    qCDebug(TOKODON_HTTP) << "DELETE" << url << "(multipart-message)";
+
+    QNetworkReply *reply = m_qnam->deleteResource(request);
+    reply->setParent(parent);
+    handleReply(reply, callback);
+}
+
 QNetworkRequest Account::makeRequest(const QUrl &url, bool authenticated) const
 {
     QNetworkRequest request(url);
@@ -122,15 +151,18 @@ QNetworkRequest Account::makeRequest(const QUrl &url, bool authenticated) const
     return request;
 }
 
-void Account::handleReply(QNetworkReply *reply, std::function<void(QNetworkReply *)> reply_cb) const
+void Account::handleReply(QNetworkReply *reply, std::function<void(QNetworkReply *)> reply_cb, std::function<void(QNetworkReply *)> errorCallback) const
 {
-    QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb]() {
+    connect(reply, &QNetworkReply::finished, [reply, reply_cb, errorCallback]() {
         reply->deleteLater();
-        if (reply_cb) {
-            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) && !reply->url().toString().contains("nodeinfo")) {
-                qCWarning(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << reply->url();
-                return;
+        if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) && !reply->url().toString().contains("nodeinfo")) {
+            qCWarning(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << reply->url();
+            if (errorCallback) {
+                errorCallback(reply);
             }
+            return;
+        }
+        if (reply_cb) {
             reply_cb(reply);
         }
     });
@@ -142,13 +174,17 @@ void Account::handleReply(QNetworkReply *reply, std::function<void(QNetworkReply
 }
 
 // assumes file is already opened and named
-void Account::upload(Post *p, QFile *file, const QString &filename)
+QNetworkReply *Account::upload(const QUrl &filename, std::function<void(QNetworkReply *)> callback)
 {
+    auto file = new QFile(filename.toLocalFile());
+    const QFileInfo info(filename.toLocalFile());
+    file->open(QFile::ReadOnly);
+
     auto mp = new QHttpMultiPart(QHttpMultiPart::FormDataType);
 
     QHttpPart filePart;
     filePart.setHeader(QNetworkRequest::ContentTypeHeader, "application/octet-stream");
-    filePart.setHeader(QNetworkRequest::ContentDispositionHeader, QString("attachment; name=\"file\"; filename=\"%1\"").arg(filename));
+    filePart.setHeader(QNetworkRequest::ContentDispositionHeader, QStringLiteral("attachment; name=\"file\"; filename=\"%1\"").arg(info.fileName()));
     filePart.setBodyDevice(file);
     file->setParent(mp);
 
@@ -157,16 +193,7 @@ void Account::upload(Post *p, QFile *file, const QString &filename)
     const auto uploadUrl = apiUrl("/api/v1/media");
     qCDebug(TOKODON_HTTP) << "POST" << uploadUrl << "(upload)";
 
-    post(uploadUrl, mp, true, this, [=](QNetworkReply *reply) {
-        const auto data = reply->readAll();
-        const auto doc = QJsonDocument::fromJson(data);
-
-        if (!doc.isObject()) {
-            return;
-        }
-
-        p->addAttachment(doc.object());
-    });
+    return post(uploadUrl, mp, true, this, callback);
 }
 
 static QMap<QString, AbstractAccount::StreamingEventType> stringToStreamingEventType = {
@@ -225,14 +252,20 @@ void Account::validateToken()
             return;
         }
 
-        const auto data = reply->readAll();
-        const auto doc = QJsonDocument::fromJson(data);
+        const auto doc = QJsonDocument::fromJson(reply->readAll());
 
-        if (!doc.isObject() || !doc.object().contains("source")) {
+        if (!doc.isObject()) {
             return;
         }
 
-        m_identity.fromSourceData(doc.object());
+        const auto object = doc.object();
+        if (!object.contains("source")) {
+            return;
+        }
+
+        m_identity = identityLookup(object["id"].toString(), object);
+        m_name = m_identity->username();
+        Q_EMIT identityChanged();
         Q_EMIT authenticated();
     });
 
@@ -242,9 +275,14 @@ void Account::validateToken()
     streamingSocket("user");
 }
 
+QString Account::settingsGroupName() const
+{
+    return m_name + QLatin1Char('@') + QUrl(m_instance_uri).host();
+}
+
 void Account::writeToSettings(QSettings &settings) const
 {
-    settings.beginGroup(m_name);
+    settings.beginGroup(settingsGroupName());
 
     settings.setValue("token", m_token);
     settings.setValue("client_id", m_client_id);
@@ -267,4 +305,4 @@ void Account::buildFromSettings(const QSettings &settings)
         m_ignoreSslErrors = settings.value("ignoreSslErrors", false).toBool();
         validateToken();
     }
-}
+}
\ No newline at end of file
