@@ -12,14 +12,13 @@
 
 class AbstractAccount;
 class QNetworkAccessManager;
-class AccountModel;
 
 class AccountManager : public QAbstractListModel
 {
     Q_OBJECT
-    Q_PROPERTY(bool hasAccounts READ hasAccounts NOTIFY accountAdded NOTIFY accountRemoved)
+    Q_PROPERTY(bool hasAccounts READ hasAccounts NOTIFY accountsChanged)
     Q_PROPERTY(AbstractAccount *selectedAccount READ selectedAccount WRITE selectAccount NOTIFY accountSelected)
-    Q_PROPERTY(QVariant selectedAccountModel READ selectedAccountModel NOTIFY accountSelected)
+    Q_PROPERTY(QString selectedAccountId READ selectedAccountId NOTIFY accountSelected)
     Q_PROPERTY(int selectedIndex READ selectedIndex NOTIFY accountSelected)
     Q_PROPERTY(KAboutData aboutData READ aboutData WRITE setAboutData NOTIFY aboutDataChanged)
 public:
@@ -40,7 +39,7 @@ public:
 
     void selectAccount(AbstractAccount *account, bool explicitUserAction = true);
     AbstractAccount *selectedAccount() const;
-    QVariant selectedAccountModel();
+    QString selectedAccountId() const;
 
     int selectedIndex() const;
 
@@ -51,11 +50,12 @@ public:
     QVariant data(const QModelIndex &index, int role) const override;
     QHash<int, QByteArray> roleNames() const override;
 
-    Q_INVOKABLE AbstractAccount *createNewAccount(const QString &username, const QString &instanceUri, bool ignoreSslErrors = false);
+    Q_INVOKABLE AbstractAccount *createNewAccount(const QString &instanceUri, bool ignoreSslErrors = false);
 
 Q_SIGNALS:
     void accountAdded(AbstractAccount *account);
     void accountRemoved(AbstractAccount *account);
+    void accountsChanged();
     void accountSelected(AbstractAccount *account);
     void identityChanged(AbstractAccount *account);
     void fetchedTimeline(AbstractAccount *account, QString original_name, QList<Post *> posts);
