@@ -16,6 +16,7 @@ class Identity : public QObject
 {
     Q_OBJECT
 
+    Q_PROPERTY(QString id READ id NOTIFY identityUpdated)
     Q_PROPERTY(QString displayName READ displayName NOTIFY identityUpdated)
     Q_PROPERTY(QString displayNameHtml READ displayNameHtml NOTIFY identityUpdated)
     Q_PROPERTY(QString username READ username NOTIFY identityUpdated)
@@ -33,7 +34,7 @@ class Identity : public QObject
     Q_PROPERTY(Relationship *relationship READ relationship NOTIFY relationshipChanged)
 
 public:
-    qint64 id() const;
+    QString id() const;
     QString displayName() const;
     QString username() const;
     QString displayNameHtml() const;
@@ -61,7 +62,7 @@ Q_SIGNALS:
     void identityUpdated();
 
 private:
-    qint64 m_id;
+    QString m_id;
     QString m_displayName;
     QString m_displayNameHtml;
     QString m_username;
