@@ -32,23 +32,32 @@
 
 #include "account.h"
 #include "accountmanager.h"
+#include "accountmodel.h"
 #include "attachmenteditormodel.h"
 #include "blurhashimageprovider.h"
 #include "clipboard.h"
 #include "config.h"
 #include "conversationmodel.h"
 #include "filehelper.h"
+#include "linkpaginatedtimelinemodel.h"
 #include "maintimelinemodel.h"
 #include "networkaccessmanagerfactory.h"
 #include "networkcontroller.h"
+#include "networkrequestprogress.h"
 #include "notificationmodel.h"
 #include "poll.h"
 #include "post.h"
+#include "posteditorbackend.h"
 #include "profileeditor.h"
 #include "searchmodel.h"
 #include "tagsmodel.h"
+#include "threadmodel.h"
 #include "timelinemodel.h"
 
+#ifdef HAVE_COLORSCHEME
+#include "colorschemer.h"
+#endif
+
 #ifdef Q_OS_ANDROID
 Q_DECL_EXPORT
 #endif
@@ -69,6 +78,12 @@ int main(int argc, char *argv[])
     if (qEnvironmentVariableIsEmpty("QT_QUICK_CONTROLS_STYLE")) {
         QQuickStyle::setStyle(QStringLiteral("org.kde.desktop"));
     }
+
+    if (QIcon::themeName().contains("dark")) {
+        QIcon::setFallbackThemeName("tokodon-dark");
+    } else {
+        QIcon::setFallbackThemeName("tokodon");
+    }
 #endif
 
 #ifdef Q_OS_WINDOWS
@@ -105,28 +120,44 @@ int main(int argc, char *argv[])
 #endif
     auto config = Config::self();
 
+#ifdef HAVE_COLORSCHEME
+    ColorSchemer colorScheme;
+    qmlRegisterSingletonInstance<ColorSchemer>("org.kde.kmasto", 1, 0, "ColorSchemer", &colorScheme);
+    if (!config->colorScheme().isEmpty()) {
+        colorScheme.apply(config->colorScheme());
+    }
+#endif
+
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "Config", config);
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "Controller", &NetworkController::instance());
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "AccountManager", &AccountManager::instance());
     qmlRegisterType<MainTimelineModel>("org.kde.kmasto", 1, 0, "TimelineModel");
+    qmlRegisterType<LinkPaginationTimelineModel>("org.kde.kmasto", 1, 0, "LinkPaginationTimelineModel");
     qmlRegisterType<SearchModel>("org.kde.kmasto", 1, 0, "SearchModel");
+    qmlRegisterType<ThreadModel>("org.kde.kmasto", 1, 0, "ThreadModel");
     qmlRegisterType<ConversationModel>("org.kde.kmasto", 1, 0, "ConversationModel");
     qmlRegisterType<TagsModel>("org.kde.kmasto", 1, 0, "TagsModel");
+    qmlRegisterType<AccountModel>("org.kde.kmasto", 1, 0, "AccountModel");
     qmlRegisterType<ProfileEditorBackend>("org.kde.kmasto", 1, 0, "ProfileEditorBackend");
+    qmlRegisterType<NetworkRequestProgress>("org.kde.kmasto", 1, 0, "NetworkRequestProgress");
+    qmlRegisterType<PostEditorBackend>("org.kde.kmasto", 1, 0, "PostEditorBackend");
     qmlRegisterType<NotificationModel>("org.kde.kmasto", 1, 0, "NotificationModel");
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "Clipboard", new Clipboard);
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "FileHelper", new FileHelper);
     qmlRegisterSingletonType("org.kde.kmasto", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
         return engine->toScriptValue(KAboutData::applicationData());
     });
-    qmlRegisterType<AttachmentEditorModel>("org.kde.kmasto", 1, 0, "AttachmentEditorModel");
     qRegisterMetaType<Account *>("Account*");
     qRegisterMetaType<AbstractAccount *>("AbstractAccount*");
     qRegisterMetaType<Identity *>("Identity*");
+    qRegisterMetaType<AttachmentEditorModel *>("AttachmentEditorModel*");
     qRegisterMetaType<Poll *>("Poll*");
+    qRegisterMetaType<QNetworkReply *>("QNetworkReply*");
     qRegisterMetaType<Relationship *>("Relationship*");
     qmlRegisterUncreatableType<Post>("org.kde.kmasto", 1, 0, "Post", "ENUM");
+    qmlRegisterUncreatableType<Attachment>("org.kde.kmasto", 1, 0, "Attachment", "ENUM");
     qmlRegisterUncreatableType<Notification>("org.kde.kmasto", 1, 0, "Notification", "ENUM");
+    qmlRegisterSingletonType(QUrl("qrc:/content/ui/Navigation.qml"), "org.kde.kmasto", 1, 0, "Navigation");
 
     QQmlApplicationEngine engine;
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
