@@ -10,64 +10,95 @@
 #include "account.h"
 #include "poll.h"
 #include "post.h"
+#include "utils.h"
 
 static QMap<QString, Attachment::AttachmentType> stringToAttachmentType = {
     {"image", Attachment::AttachmentType::Image},
     {"gifv", Attachment::AttachmentType::GifV},
     {"video", Attachment::AttachmentType::Video},
+    {"unknown", Attachment::AttachmentType::Unknown},
 };
 
-Attachment::Attachment(Post *parent, const QJsonObject &obj)
-    : m_parent(parent)
+Attachment::Attachment(QObject *parent)
+    : QObject(parent)
+{
+}
+
+Attachment::Attachment(const QJsonObject &obj, QObject *parent)
+    : QObject(parent)
+{
+    fromJson(obj);
+}
+
+void Attachment::fromJson(const QJsonObject &obj)
 {
-    m_type = Unknown;
     if (!obj.contains("type")) {
+        m_type = Unknown;
         return;
     }
 
     m_id = obj["id"].toString();
     m_url = obj["url"].toString();
     m_preview_url = obj["preview_url"].toString();
-    m_description = obj["description"].toString();
+    m_remote_url = obj["remote_url"].toString();
+
+    setDescription(obj["description"].toString());
     m_blurhash = obj["blurhash"].toString();
     m_originalHeight = obj["meta"].toObject()["original"].toObject()["height"].toInt();
     m_originalWidth = obj["meta"].toObject()["original"].toObject()["width"].toInt();
 
     // determine type if we can
-    auto type = obj["type"].toString();
+    const auto type = obj["type"].toString();
     if (stringToAttachmentType.contains(type)) {
         m_type = stringToAttachmentType[type];
     }
 }
 
-Attachment::~Attachment() = default;
+QString Attachment::description() const
+{
+    return m_description;
+}
 
 void Attachment::setDescription(const QString &description)
 {
     m_description = description;
-    m_parent->updateAttachment(this);
 }
 
-static QMap<Post::Visibility, QString> visibilityToString = {
-    {Post::Visibility::Public, "public"},
-    {Post::Visibility::Unlisted, "unlisted"},
-    {Post::Visibility::Private, "private"},
-    {Post::Visibility::Direct, "direct"},
-};
-
-static QMap<QString, Post::Visibility> stringToVisibility = {
-    {"public", Post::Visibility::Public},
-    {"unlisted", Post::Visibility::Unlisted},
-    {"private", Post::Visibility::Private},
-    {"direct", Post::Visibility::Direct},
-};
+QString Attachment::id() const
+{
+    return m_id;
+}
 
 Post::Post(AbstractAccount *account, QObject *parent)
     : QObject(parent)
     , m_parent(account)
 {
-    QString visibilityString = account->identity().visibility();
-    m_visibility = stringToVisibility[visibilityString];
+    QString visibilityString = account->identity()->visibility();
+    m_visibility = stringToVisibility(visibilityString);
+}
+
+QString computeContent(const QJsonObject &obj, std::shared_ptr<Identity> authorIdentity)
+{
+    QString content = obj["content"].toString();
+    const auto emojis = obj["emojis"].toArray();
+
+    for (const auto &emoji : emojis) {
+        const auto emojiObj = emoji.toObject();
+        content = content.replace(QLatin1Char(':') + emojiObj["shortcode"].toString() + QLatin1Char(':'),
+                                  "<img height=\"16\" align=\"middle\" width=\"16\" src=\"" + emojiObj["static_url"].toString() + "\">");
+    }
+
+    const auto tags = obj["tags"].toArray();
+    const QString baseUrl = authorIdentity->url().toDisplayString(QUrl::RemovePath);
+
+    for (const auto &tag : tags) {
+        const auto tagObj = tag.toObject();
+        content = content.replace(baseUrl + QStringLiteral("/tags/") + tagObj["name"].toString(),
+                                  QStringLiteral("hashtag:/") + tagObj["name"].toString(),
+                                  Qt::CaseInsensitive);
+    }
+
+    return content;
 }
 
 Post::Post(AbstractAccount *account, QJsonObject obj, QObject *parent)
@@ -75,100 +106,99 @@ Post::Post(AbstractAccount *account, QJsonObject obj, QObject *parent)
     , m_parent(account)
     , m_visibility(Post::Visibility::Public)
 {
-    const auto account_doc = obj["account"].toObject();
-    const auto acct = account_doc["acct"].toString();
-    const auto reblog_obj = obj["reblog"].toObject();
+    fromJson(obj);
+}
 
-    m_original_post_id = obj["id"].toString();
+void Post::fromJson(QJsonObject obj)
+{
+    const auto accountDoc = obj["account"].toObject();
+    const auto accountId = accountDoc["id"].toString();
 
-    if (!obj.contains("reblog") || reblog_obj.isEmpty()) {
-        m_repeat = false;
-        m_authorIdentity = m_parent->identityLookup(acct, account_doc);
+    m_originalPostId = obj["id"].toString();
+    const auto reblogObj = obj["reblog"].toObject();
+
+    if (!obj.contains("reblog") || reblogObj.isEmpty()) {
+        m_boosted = false;
+        m_authorIdentity = m_parent->identityLookup(accountId, accountDoc);
     } else {
-        m_repeat = true;
+        m_boosted = true;
 
-        auto repeat_account_doc = reblog_obj["account"].toObject();
-        auto repeat_acct = repeat_account_doc["acct"].toString();
+        const auto reblogAccountDoc = reblogObj["account"].toObject();
+        const auto reblogAccountId = reblogAccountDoc["id"].toString();
 
-        m_authorIdentity = m_parent->identityLookup(repeat_acct, repeat_account_doc);
-        m_repeatIdentity = m_parent->identityLookup(acct, account_doc);
+        m_authorIdentity = m_parent->identityLookup(reblogAccountId, reblogAccountDoc);
+        m_boostIdentity = m_parent->identityLookup(accountId, accountDoc);
 
-        obj = reblog_obj;
+        obj = reblogObj;
     }
 
-    m_subject = obj["spoiler_text"].toString();
-    m_content = obj["content"].toString();
+    m_postId = obj["id"].toString();
 
-    const auto emojis = obj["emojis"].toArray();
+    m_spoilerText = obj["spoiler_text"].toString();
+    m_content = computeContent(obj, m_authorIdentity);
 
-    for (const auto &emoji : emojis) {
-        const auto emojiObj = emoji.toObject();
-        m_content = m_content.replace(QLatin1Char(':') + emojiObj["shortcode"].toString() + QLatin1Char(':'),
-                                      "<img height=\"16\" align=\"middle\" width=\"16\" src=\"" + emojiObj["static_url"].toString() + "\">");
-    }
+    m_replyTargetId = obj["id"].toString();
+    m_url = QUrl(obj["url"].toString());
 
-    const auto tags = obj["tags"].toArray();
-    const QString baseUrl = authorIdentity()->url().toDisplayString(QUrl::RemovePath);
-
-    for (const auto &tag : tags) {
-        const auto tagObj = tag.toObject();
-        m_content = m_content.replace(baseUrl + QStringLiteral("/tags/") + tagObj["name"].toString(),
-                                      QStringLiteral("hashtag:/") + tagObj["name"].toString(),
-                                      Qt::CaseInsensitive);
-    }
-
-    m_post_id = m_replyTargetId = obj["id"].toString();
-    m_isFavorite = obj["favourited"].toBool();
-    m_favoriteCount = obj["favourites_count"].toInt();
-    m_repeatedCount = obj["reblogs_count"].toInt();
+    m_favouritesCount = obj["favourites_count"].toInt();
+    m_reblogsCount = obj["reblogs_count"].toInt();
     m_repliesCount = obj["replies_count"].toInt();
-    m_isRepeated = obj["reblogged"].toBool();
-    m_isSensitive = obj["sensitive"].toBool();
-    m_link = QUrl(obj["url"].toString());
+
+    m_favourited = obj["favourited"].toBool();
+    m_reblogged = obj["reblogged"].toBool();
+    m_bookmarked = obj["bookmarked"].toBool();
     m_pinned = obj["pinned"].toBool();
-    m_visibility = stringToVisibility[obj["visibility"].toString()];
-    m_published_at = QDateTime::fromString(obj["created_at"].toString(), Qt::ISODate).toLocalTime();
+    m_muted = obj["muted"].toBool();
+
+    m_filters.clear();
+
+    const auto filters = obj["filtered"].toArray();
+    for (const auto &filter : filters) {
+        const auto filterContext = filter.toObject();
+        const auto filterObj = filterContext["filter"].toObject();
+        m_filters << filterObj["title"].toString();
+        m_filtered = true;
+    }
+
+    m_sensitive = obj["sensitive"].toBool();
+    m_visibility = stringToVisibility(obj["visibility"].toString());
+    m_language = obj["language"].toString();
+
+    m_publishedAt = QDateTime::fromString(obj["created_at"].toString(), Qt::ISODate).toLocalTime();
+
+    m_attachments.clear();
     addAttachments(obj["media_attachments"].toArray());
     const QJsonArray mentions = obj["mentions"].toArray();
     if (obj.contains("card") && !obj["card"].toObject().empty()) {
         setCard(std::make_optional<Card>(obj["card"].toObject()));
     }
 
+    m_mentions.clear();
     for (const auto &m : qAsConst(mentions)) {
         const QJsonObject o = m.toObject();
         m_mentions.push_back("@" + o["acct"].toString());
     }
 
     if (obj.contains(QStringLiteral("poll")) && !obj[QStringLiteral("poll")].isNull()) {
-        m_poll = new Poll(obj[QStringLiteral("poll")].toObject());
+        m_poll = std::make_unique<Poll>(obj[QStringLiteral("poll")].toObject());
     }
 
-    m_attachments_visible = !m_isSensitive;
-}
-
-Post::~Post()
-{
-    qDeleteAll(m_attachments);
-    delete m_poll;
+    m_attachments_visible = !m_sensitive;
 }
 
 void Post::addAttachments(const QJsonArray &attachments)
 {
-    for (const auto &attachment_val : attachments) {
-        auto attachment_obj = attachment_val.toObject();
-        auto attachment = new Attachment(this, attachment_obj);
-
-        m_attachments.push_back(attachment);
+    for (const auto &attachment : attachments) {
+        m_attachments.append(new Attachment{attachment.toObject(), this});
     }
 }
 
 void Post::addAttachment(const QJsonObject &attachment)
 {
-    auto att = new Attachment(this, attachment);
+    auto att = new Attachment{attachment, this};
     if (att->m_url.isEmpty()) {
         return;
     }
-
     m_attachments.append(att);
 
     Q_EMIT attachmentUploaded();
@@ -203,6 +233,16 @@ QStringList Post::mentions() const
     return m_mentions;
 }
 
+QStringList Post::filters() const
+{
+    return m_filters;
+}
+
+QUrl Post::url() const
+{
+    return m_url;
+}
+
 void Post::setMentions(const QStringList &mentions)
 {
     if (mentions == m_mentions) {
@@ -212,42 +252,19 @@ void Post::setMentions(const QStringList &mentions)
     Q_EMIT mentionsChanged();
 }
 
-QJsonDocument Post::toJsonDocument() const
+QDateTime Post::publishedAt() const
 {
-    QJsonObject obj;
-
-    obj["spoiler_text"] = m_subject;
-    obj["status"] = m_content;
-    obj["content_type"] = m_content_type;
-    obj["sensitive"] = m_isSensitive;
-    obj["visibility"] = visibilityToString[m_visibility];
-
-    if (!m_replyTargetId.isEmpty()) {
-        obj["in_reply_to_id"] = m_replyTargetId;
-    }
-
-    auto media_ids = QJsonArray();
-    for (const auto att : qAsConst(m_attachments)) {
-        media_ids.append(att->m_id);
-    }
-
-    obj["media_ids"] = media_ids;
-
-    return QJsonDocument(obj);
+    return m_publishedAt;
 }
 
-void Post::uploadAttachment(const QUrl &filename)
+int Post::favouritesCount() const
 {
-    QFile *file = new QFile(filename.toLocalFile());
-    const QFileInfo info(filename.toLocalFile());
-
-    file->open(QFile::ReadOnly);
-    m_parent->upload(this, file, info.fileName());
+    return m_favouritesCount;
 }
 
-void Post::updateAttachment(Attachment *a)
+int Post::reblogsCount() const
 {
-    m_parent->updateAttachment(a);
+    return m_reblogsCount;
 }
 
 static QMap<QString, Notification::Type> str_to_not_type = {
@@ -256,18 +273,19 @@ static QMap<QString, Notification::Type> str_to_not_type = {
     {"mention", Notification::Type::Mention},
     {"reblog", Notification::Type::Repeat},
     {"update", Notification::Type::Update},
+    {"poll", Notification::Type::Poll},
 };
 
 Notification::Notification(AbstractAccount *account, const QJsonObject &obj, QObject *parent)
     : m_account(account)
 {
-    const QJsonObject accountObj = obj["account"].toObject();
-    const QJsonObject status = obj["status"].toObject();
-    auto acct = accountObj["acct"].toString();
-    auto type = obj["type"].toString();
+    const auto accountObj = obj["account"].toObject();
+    const auto status = obj["status"].toObject();
+    const auto accountId = accountObj["id"].toString();
+    const auto type = obj["type"].toString();
 
     m_post = new Post(m_account, status, parent);
-    m_identity = m_account->identityLookup(acct, accountObj);
+    m_identity = m_account->identityLookup(accountId, accountObj);
     m_type = str_to_not_type[type];
     m_id = obj["id"].toString().toInt();
 }
@@ -297,18 +315,18 @@ std::shared_ptr<Identity> Notification::identity() const
     return m_identity;
 }
 
-QString Post::subject() const
+QString Post::spoilerText() const
 {
-    return m_subject;
+    return m_spoilerText;
 }
 
-void Post::setSubject(const QString &subject)
+void Post::setSpoilerText(const QString &spoilerText)
 {
-    if (subject == m_subject) {
+    if (spoilerText == m_spoilerText) {
         return;
     }
-    m_subject = subject;
-    Q_EMIT subjectChanged();
+    m_spoilerText = spoilerText;
+    Q_EMIT spoilerTextChanged();
 }
 
 QString Post::content() const
@@ -339,17 +357,17 @@ void Post::setContentType(const QString &contentType)
     Q_EMIT contentTypeChanged();
 }
 
-bool Post::isSensitive() const
+bool Post::sensitive() const
 {
-    return m_isSensitive;
+    return m_sensitive;
 }
 
-void Post::setSensitive(bool isSensitive)
+void Post::setSensitive(bool sensitive)
 {
-    if (m_isSensitive == isSensitive) {
+    if (m_sensitive == sensitive) {
         return;
     }
-    m_isSensitive = isSensitive;
+    m_sensitive = sensitive;
     Q_EMIT sensitiveChanged();
 }
 
@@ -367,6 +385,20 @@ void Post::setVisibility(Visibility visibility)
     Q_EMIT visibilityChanged();
 }
 
+QString Post::language() const
+{
+    return m_language;
+}
+
+void Post::setLanguage(const QString &language)
+{
+    if (language == m_language) {
+        return;
+    }
+    m_language = language;
+    Q_EMIT languageChanged();
+}
+
 std::optional<Card> Post::card() const
 {
     return m_card;
@@ -377,6 +409,81 @@ void Post::setCard(std::optional<Card> card)
     m_card = card;
 }
 
+bool Post::favourited() const
+{
+    return m_favourited;
+}
+
+void Post::setFavourited(bool favourited)
+{
+    m_favourited = favourited;
+}
+
+bool Post::reblogged() const
+{
+    return m_reblogged;
+}
+
+void Post::setReblogged(bool reblogged)
+{
+    m_reblogged = reblogged;
+}
+
+bool Post::muted() const
+{
+    return m_muted;
+}
+
+void Post::setMuted(bool muted)
+{
+    m_muted = muted;
+}
+
+bool Post::bookmarked() const
+{
+    return m_bookmarked;
+}
+
+void Post::setBookmarked(bool bookmarked)
+{
+    m_bookmarked = bookmarked;
+}
+
+bool Post::pinned() const
+{
+    return m_pinned;
+}
+
+void Post::setPinned(bool pinned)
+{
+    m_pinned = pinned;
+}
+
+bool Post::filtered() const
+{
+    return m_filtered;
+}
+
+QVector<Attachment *> Post::attachments() const
+{
+    return m_attachments;
+}
+
+void Post::setAttachmentsVisible(bool attachmentsVisible)
+{
+    m_attachments_visible = attachmentsVisible;
+}
+
+bool Post::attachmentsVisible() const
+{
+    return m_attachments_visible;
+}
+
+bool Post::boosted() const
+{
+    return m_boosted;
+}
+
 Card::Card(QJsonObject card)
     : m_card(card)
 {
@@ -456,23 +563,33 @@ std::shared_ptr<Identity> Post::authorIdentity() const
     return m_authorIdentity;
 }
 
-std::shared_ptr<Identity> Post::repeatIdentity() const
+std::shared_ptr<Identity> Post::boostIdentity() const
 {
-    return m_repeatIdentity;
+    return m_boostIdentity;
 }
 
 Poll *Post::poll() const
 {
-    return m_poll;
+    return m_poll.get();
 }
 
-void Post::setPoll(Poll *poll)
+void Post::setPollJson(const QJsonObject &object)
 {
-    m_poll = poll;
+    m_poll = std::make_unique<Poll>(object);
     Q_EMIT pollChanged();
 }
 
 QString Post::postId() const
 {
-    return m_post_id;
+    return m_postId;
+}
+
+QString Post::originalPostId() const
+{
+    return m_originalPostId;
+}
+
+bool Post::isEmpty() const
+{
+    return m_postId.isEmpty();
 }
